/*
** SpeciesSet.c
*/

# include <stdlib.h>
# include <stdio.h>
# include <string.h>
# include "SpeciesSet.h"

/*@only@*/ SpeciesSet SpeciesSet_new () {
    SpeciesSet set = malloc (sizeof (*set));

    if (set == NULL) { 
      fprintf (stderr, "Unable to create new SpeciesSet.  Out of memory in SpeciesSet_new.");
      exit (EXIT_FAILURE); 
    }
    
    set->first = NULL;
    return set;
}

void SpeciesSet_free (/*@only@*/ SpeciesSet set) {
  SpeciesSetNode node = set->first;
  
  while (node != NULL) {
    SpeciesSetNode next = node->next;
    free (node);
    node = next;
  }
  
  free (set);
}

bool SpeciesSet_contains (SpeciesSet set, const char *name) {
  SpeciesSetNode node;
  
  for (node = set->first; node != NULL; node=node->next) {
    if (strcmp (name, Species_getName (node->member)) == 0) {
      return TRUE;
    }
  }

  return FALSE;
}

void SpeciesSet_insert (SpeciesSet set, /*@dependent@*/ Species species) {
  SpeciesSetNode node = malloc (sizeof (*node));

  if (node == NULL) {
    fprintf (stderr, "Unable to create new SpeciesSet.  Out of memory in SpeciesSet_insert.");
    exit (EXIT_FAILURE); 
  }
  
  node->member = species;
  node->next = set->first;
  set->first = node;
}

/* Private operation (static means this is not visible outside SpeciesSet.c */
static /*@null@*/ SpeciesSetNode SpeciesSet_removeHelper (/*@null@*/ /*@only@*/ SpeciesSetNode first,
							  Species species) {
  if (first == NULL) {
    return NULL;
  } else if /*@access Species@*/ (first->member == species) /*@noaccess Species@*/ {
    SpeciesSetNode temp;
    temp = first->next;
    free (first);
    return temp;
  } else {
    first->next = SpeciesSet_removeHelper (first->next, species);
    return first;
  }
}

void SpeciesSet_remove (SpeciesSet set, Species species) {
  set->first = SpeciesSet_removeHelper (set->first, species);
}

SpeciesSet SpeciesSet_copy (SpeciesSet /*@temp@*/ set) {
  SpeciesSetNode node;
  
  SpeciesSet copy = SpeciesSet_new ();
  for (node = set->first; node != NULL; node = node->next) {
    SpeciesSet_insert (copy, node->member);
  }

  return copy;
}

void SpeciesSet_split (SpeciesSet orig, unsigned bits,
		       SpeciesSet left, SpeciesSet right) 
{
  SpeciesSetNode node;

  for (node = orig->first; node != NULL; node = node->next) {
    if ((bits & 1) == 1U) {
      SpeciesSet_insert (left, node->member);
    } else {
      SpeciesSet_insert (right, node->member);
    }
    bits >>= 1;
  }
}

unsigned SpeciesSet_size (SpeciesSet /*@temp@*/ set) {
    unsigned size = 0;
    SpeciesSetNode node;

    for (node = set->first; node != NULL; node = node->next) {
      size++;
    }

    return size;
}

bool SpeciesSet_isEmpty (SpeciesSet set) {
  return (set->first == NULL);
}
